{-|
Module      : OpenAlex.Client
Description : Textmining Collaborative Platform
Copyright   : (c) CNRS, 2023-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

@Gargantext@: search, map, share
-}

module OpenAlex.Client where

import Protolude
import Servant.API
import Servant.Client

import OpenAlex.Types (Page, PerPage, Cursor, Filter, Search, ListOf(..), Concept, Work)

type API_URL = Text
apiUrl :: API_URL

apiUrl = "api.openalex.org"

type PageParams =
     QueryParam "page" Page
  :> QueryParam "per-page" PerPage
  :> QueryParam "cursor" Cursor

type OpenAlexAPI =
  -- https://api.openalex.org/concepts
  "concepts"
  :> QueryParam "page" Page
  :> QueryParam "per-page" PerPage
  :> QueryParam "cursor" Cursor
  :> QueryParam "filter" Filter
  :> QueryParam "search" Search
  -- TODO: filter, search, sort
  :> Get '[JSON] (ListOf Concept)

  -- https://docs.openalex.org/api-entities/works
  :<|> "works"
  :> QueryParam "page" Page
  :> QueryParam "per-page" PerPage
  :> QueryParam "cursor" Cursor
  :> QueryParam "filter" Filter
  :> QueryParam "search" Search
  -- TODO: filter, search, sort
  :> Get '[JSON] (ListOf Work)

openAlexApi :: Proxy OpenAlexAPI
openAlexApi = Proxy

concepts :: Maybe Page
         -> Maybe PerPage
         -> Maybe Cursor
         -> Maybe Filter
         -> Maybe Search
         -> ClientM (ListOf Concept)

works :: Maybe Page
      -> Maybe PerPage
      -> Maybe Cursor
      -> Maybe Filter
      -> Maybe Search
      -> ClientM (ListOf Work)

concepts :<|> works = client openAlexApi
